<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Gateway\HirePurchase\Request;

use Magento\Payment\Gateway\ConfigInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;

/**
 * @inheritdoc
 */
class StatusRequest implements BuilderInterface
{
    /**
     * @var ConfigInterface
     */
    private $config;

    /**
     *
     * @param \Magento\Payment\Gateway\ConfigInterface $config
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Magento\Payment\Gateway\Data\PaymentDataObjectFactory $paymentDataObjectFactory
     */
    public function __construct(
        \Magento\Payment\Gateway\ConfigInterface $config,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Payment\Gateway\Data\PaymentDataObjectFactory $paymentDataObjectFactory
    ) {
        $this->config = $config;
        $this->urlInterface = $urlInterface;
        $this->paymentDataObjectFactory = $paymentDataObjectFactory;
    }

    /**
     * Builds the request
     *
     * @param array $buildSubject
     * @return array
     */
    public function build(array $buildSubject)
    {

        if (!isset($buildSubject['payment'])) {
            if (!$buildSubject['payment'] instanceof PaymentDataObjectInterface &&
                is_object($buildSubject['payment'])
            ) {
                try {
                    $buildSubject['payment'] = $this->paymentDataObjectFactory->create($buildSubject['payment']);
                } catch (\Exception $e) {
                    throw new \InvalidArgumentException(__($e->getMessage()));
                }
            } else {
                throw new \InvalidArgumentException(__('Payment data object should be provided'));
            }
        }

        /** @var PaymentDataObjectInterface $payment */
        $payment = $buildSubject['payment'];
        $order = $payment->getOrder();
        $storeId = $order->getStoreId();
        $environment = $this->config->getValue('environment', $storeId);
        $orderIncrementId = $order->getOrderIncrementId() ?
                    $order->getOrderIncrementId() : $order->getData('increment_id');
        $privKey = $this->config->getValue($environment.'_private_key', $storeId);
        $requestParams = '/applications?merchant_reference='.$orderIncrementId;
        $statusRequestUrl = $this->config->getValue($environment.'_api_url', $storeId).$requestParams;

        $headers = [
            'X-Access-Key' => $this->config->getValue($environment.'_public_key', $storeId),
            'X-Signature' => $this->config->generateSignature($requestParams, $privKey, false),
        ];

        return [
            'url' => $statusRequestUrl,
            'headers' => $headers,
            'method' => 'GET'
        ];
    }
}
