<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\Handler;

use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderManagementInterface;
use Montonio\Hirepurchase\Gateway\Payments\Config\Config as PaymentsConfig;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\ResponseHandlerInterface;

/**
 * Handler responsible for client notification.
 */
class ClientNotificationHandler implements ResponseHandlerInterface
{
    /**
     * @var OrderManagementInterface
     */
    private $orderManagement;

    /**
     * @var PaymentsConfig
     */
    private $paymentsConfig;

    /**
     * @param OrderManagementInterface $orderManagement
     * @param PaymentsConfig $paymentsConfig
     */
    public function __construct(
        OrderManagementInterface $orderManagement,
        PaymentsConfig $paymentsConfig
    ) {
        $this->orderManagement = $orderManagement;
        $this->paymentsConfig = $paymentsConfig;
    }

    /**
     * @inheritdoc
     */
    public function handle(OrderInterface $order, array $data): void
    {
        if ((bool)$this->paymentsConfig->getValue('order_email_after') === true &&
            (bool)$order->getEmailSent() === false
        ) {
            $result = (bool)$this->orderManagement->notify($order->getEntityId());

            if ($result === true) {
                $order->addStatusHistoryComment(__('Order confirmation e-mail sent to customer'));
                $order->setEmailSent(true);
            }
        }
    }
}
