<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\Handler;

use Magento\Payment\Model\MethodInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\ResponseHandlerInterface;

/**
 * Finalized payment status response handler
 */
class FinalizedOrderStatusHandler implements ResponseHandlerInterface
{
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var ResponseHandlerInterface[]
     */
    private $finalizedOrderHandlers;

    /**
     * @param OrderRepositoryInterface $orderRepository
     * @param ResponseHandlerInterface[] $finalizedOrderHandlers
     */
    public function __construct(
        OrderRepositoryInterface $orderRepository,
        array $finalizedOrderHandlers = []
    ) {
        $this->orderRepository = $orderRepository;
        $this->finalizedOrderHandlers = $finalizedOrderHandlers;
    }

    /**
     * @inheritdoc
     */
    public function handle(OrderInterface $order, array $data): void
    {
        //only try once when order state new and status pending
        if ($order->getState() === Order::STATE_NEW && $order->getStatus() === 'pending') {
            $this->updateOrderStatus($order, $data);

            foreach ($this->finalizedOrderHandlers as $handler) {
                $handler->handle($order, $data);
            }

            $this->orderRepository->save($order);
        }
    }

    /**
     * Update order status.
     *
     * @param OrderInterface $order
     * @param array $data
     *
     * @return void
     */
    private function updateOrderStatus(OrderInterface $order, array $data): void
    {
        /* @var MethodInterface $method */
        $method = $order->getPayment()->getMethodInstance();

        $order->setState(Order::STATE_PROCESSING);
        $order->setStatus(Order::STATE_PROCESSING);
        $order->addStatusHistoryComment(
            __(
                'Status update by customer - method: %1, amount: %2',
                $method->getCode(),
                $data['amount']
            )
        );
    }
}
