<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\ResponseHandler;

/**
 * Class responsible for providing response handler.
 */
class HandlerPool
{
    /**
     * Undefined handler code constant.
     */
    private const UNKNOWN_STATUS_HANDLER_CODE = 'unknown';

    /**
     * @var ResponseHandlerInterface[]
     */
    private $handlers;

    /**
     * @param ResponseHandlerInterface[] $handlers
     */
    public function __construct(
        array $handlers = []
    ) {
        $this->handlers = $handlers;
    }

    /**
     * Retrieve response handler by response status.
     *
     * @param string $status
     *
     * @return ResponseHandlerInterface
     */
    public function getHandlerByStatus(string $status): ResponseHandlerInterface
    {
        if (!isset($this->handlers[$status])) {
            $status = self::UNKNOWN_STATUS_HANDLER_CODE;
        }

        return $this->handlers[$status];
    }
}
