<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\Validator;

use \Magento\Payment\Gateway\Validator\AbstractValidator;
use \Magento\Payment\Gateway\Validator\ResultInterfaceFactory;
use Montonio\Hirepurchase\Gateway\Payments\Config\Config;
use Montonio\Hirepurchase\Model\Payment\PaymentsConfigProvider;

/**
 * @inheritdoc
 */
class CountryValidator extends AbstractValidator
{
    /**
     * @var Config
     */
    private $config;

    /**
     * @inheritdoc
     */
    public function __construct(
        ResultInterfaceFactory $resultFactory,
        Config $config
    ) {
        $this->config = $config;
        parent::__construct($resultFactory);
    }

    /**
     * @inheritdoc
     */
    public function validate(array $validationSubject)
    {
        $isValid = true;
        $storeId = $validationSubject['storeId'];

        $isGrouped = (int)$this->config->getValue('grouped');
        $allowSpecific = (int)$this->config->getValue('allowspecific', $storeId);
        $specificCountry = $this->config->getValue('specificcountry', $storeId);

        if ($isGrouped === 1 &&
            $allowSpecific === 1
        ) {
            $availableCountries = explode(',', $specificCountry);
            if (!in_array($validationSubject['country'], $availableCountries)) {
                $isValid =  false;
            }
        }

        if ($isGrouped === 0) {
            $methodConfigs = $this->config->getAvailableMethods();

            foreach ($methodConfigs as $code => $value) {
                $subCode = $this->config->getCode() . '_' . $code;

                if ($subCode == $validationSubject['methodCode']) {
                    if ($validationSubject['country'] == $value['method_country'] ||
                        $value['method_country'] == ''
                    ) {
                        $isValid = true;
                        break;
                    }
                }

                $isValid =  false;
            }
        }

        return $this->createResult($isValid);
    }
}
