<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Model\Method;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\DataObject;
use Magento\Framework\Event\ManagerInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Payment\Gateway\Command\CommandManagerInterface;
use Magento\Payment\Gateway\Command\CommandPoolInterface;
use Magento\Payment\Gateway\Config\ValueHandlerPoolInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectFactory;
use Magento\Payment\Gateway\Validator\ValidatorPoolInterface;
use Magento\Payment\Model\InfoInterface;
use Magento\Payment\Model\MethodInterface;
use Magento\Payment\Observer\AbstractDataAssignObserver;
use Magento\Quote\Api\Data\CartInterface;
use Psr\Log\LoggerInterface;

/**
 * @inheritdoc
 */
class SplitAdapter extends \Magento\Payment\Model\Method\Adapter
{
    /**
     * @var ValueHandlerPoolInterface
     */
    private $valueHandlerPool;

    /**
     * @var ValidatorPoolInterface
     */
    private $validatorPool;

    /**
     * @var CommandPoolInterface
     */
    private $commandPool;

    /**
     * @var int
     */
    private $storeId;

    /**
     * @var string
     */
    private $formBlockType;

    /**
     * @var string
     */
    private $infoBlockType;

    /**
     * @var InfoInterface
     */
    private $infoInstance;

    /**
     * @var string
     */
    private $code;

    /**
     * @var ManagerInterface
     */
    private $eventManager;

    /**
     * @var PaymentDataObjectFactory
     */
    private $paymentDataObjectFactory;

    /**
     * @var \Magento\Payment\Gateway\Command\CommandManagerInterface
     */
    private $commandExecutor;

    /**
     * Logger for exception details
     *
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param ManagerInterface $eventManager
     * @param ValueHandlerPoolInterface $valueHandlerPool
     * @param PaymentDataObjectFactory $paymentDataObjectFactory
     * @param string $code
     * @param string $formBlockType
     * @param string $infoBlockType
     * @param CommandPoolInterface|null $commandPool
     * @param ValidatorPoolInterface|null $validatorPool
     * @param CommandManagerInterface|null $commandExecutor
     * @param LoggerInterface|null $logger
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        ManagerInterface $eventManager,
        ValueHandlerPoolInterface $valueHandlerPool,
        PaymentDataObjectFactory $paymentDataObjectFactory,
        $code,
        $formBlockType,
        $infoBlockType,
        CommandPoolInterface $commandPool = null,
        ValidatorPoolInterface $validatorPool = null,
        CommandManagerInterface $commandExecutor = null,
        LoggerInterface $logger = null
    ) {
            $this->valueHandlerPool = $valueHandlerPool;
            $this->validatorPool = $validatorPool;
            $this->commandPool = $commandPool;
            $this->code = $code;
            $this->infoBlockType = $infoBlockType;
            $this->formBlockType = $formBlockType;
            $this->eventManager = $eventManager;
            $this->paymentDataObjectFactory = $paymentDataObjectFactory;
            $this->commandExecutor = $commandExecutor;
            $this->logger = $logger ?: ObjectManager::getInstance()->get(LoggerInterface::class);

            parent::__construct(
                $eventManager,
                $valueHandlerPool,
                $paymentDataObjectFactory,
                $code,
                $formBlockType,
                $infoBlockType,
                $commandPool,
                $validatorPool,
                $commandExecutor,
                $logger
            );
    }

    /**
     * Build Payment request command exposed for payment object
     *
     * @param array $arguments
     * @return mixed
     */
    public function buildPaymentRequest(array $arguments = [])
    {
        //implemented a custom command not handled by default payment flow
        $command = $this->commandPool->get('buildPaymentRequest');
        return $command->execute($arguments);
    }

    /**
     *
     * @param array $arguments
     * @return array
     */
    public function aspspsRequest(array $arguments = [])
    {
        //implemented a custom command not handled by default payment flow
        $command = $this->commandPool->get('aspspsRequest');
        return $command->execute($arguments);
    }

    /**
     * @inheritdoc
     */
    public function canUseForCountry($country)
    {
        try {
            $validator = $this->getValidatorPool()->get('country');
        } catch (\Exception $e) {
            return true;
        }

        //adding validationsubject with real method code
        $validationSubject = [
                        'country' => $country,
                        'storeId' => $this->getStore(),
                        'methodCode' => $this->getCode()
                ];

        $result = $validator->validate($validationSubject);

        return $result->isValid();
    }
}
