<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Plugin\Controller\Onepage;

/**
 * Handle around plugin on success page
 */
class SplitSuccess
{
    /**
     *
     * @param \Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Montonio\Hirepurchase\Model\Payment\SplitConfigProvider $paymentsConfigProvider
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory,
        \Magento\Framework\UrlInterface $urlInterface,
        \Montonio\Hirepurchase\Model\Payment\SplitConfigProvider $paymentsConfigProvider,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->resultRedirectFactory = $resultRedirectFactory;
        $this->urlInterface = $urlInterface;
        $this->paymentsConfigProvider = $paymentsConfigProvider;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     *
     * @param \Magento\Checkout\Controller\Onepage\Success $subject
     * @param callable $proceed
     * @return callable | resultRedirect
     */
    public function afterExecute(\Magento\Checkout\Controller\Onepage\Success $subject, $result)
    {

        /**
         * wondeing why this is in plugin? It's for covering checkout extensions compatibility
         * each controller you create generates new checkout flow that most checkout extensions fail to cover
         */

        $methodCode = $this->paymentsConfigProvider::CODE;

        $enabled = $this->paymentsConfigProvider->getConfigObject()->isActive();

        if (!$enabled) {
            return $result;
        }

        $default = true;

        //idea here is to get the data from existing objects to reduce load and db usage
        //if we get our data then we can redirect to payment gateway instead of success page
        $lastRealOrder = $subject
                ->getOnePage()
                ->getCheckout()
                ->getLastRealOrder() ?? false;

        $methodInstance = false;

        if (is_object($lastRealOrder->getPayment())) {
            $methodInstance = $lastRealOrder
            ->getPayment()
            ->getMethodInstance() ?? false;
        }

        $methodTarget = $this->paymentsConfigProvider::CODE;

        if ((is_object($lastRealOrder) && $lastRealOrder->getStatus() === "pending") &&
            is_object($methodInstance)
        ) {

            $methodTarget = $this->paymentsConfigProvider::CODE;
            $methodCode = $methodInstance->getCode() ?? false;
            $methodsAvailable = $this->paymentsConfigProvider->getConfigObject()->getAvailableMethods();
            $subMethodCode = str_replace($methodTarget.'_', '', $methodCode);

            if ($methodCode && isset($methodsAvailable[$subMethodCode])) {
                $requestUrl = $lastRealOrder
                    ->getPayment()
                    ->getMethodInstance()
                    ->buildPaymentRequest(
                        ['order'=> $lastRealOrder, 'method' => $subMethodCode]
                    );

                if ($requestUrl) {

                    $result = $this->resultRedirectFactory
                        ->create()->setUrl($requestUrl);
                    //in all other cases we fall back to default action
                    $default = false;
                }
            }
        }

        return $result;
    }
}
